<?php /*

 ocPortal
 Copyright (c) ocProducts, 2004-2012

 See text/EN/licence.txt for full licencing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license		http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright	ocProducts Ltd
 * @package		flagrant
 */

require_code('aed_module');

/**
 * Module page class.
 */
class Module_admin_flagrant extends standard_aed_module
{
	var $lang_type='FLAGRANT';
	var $special_edit_frontend=true;
	var $redirect_type='ed';
	var $menu_label='FLAGRANT_TEXT';
	var $select_name='MESSAGE';
	var $table='text';
	var $orderer='the_message';
	var $title_is_multi_lang=true;

	/**
	 * Standard modular info function.
	 *
	 * @return ?array	Map of module info (NULL: module is disabled).
	 */
	function info()
	{
		$info=array();
		$info['author']='Chris Graham';
		$info['organisation']='ocProducts';
		$info['hacked_by']=NULL;
		$info['hack_version']=NULL;
		$info['update_require_upgrade']=1;
		$info['version']=3;
		$info['locked']=false;
		return $info;
	}

	/**
	 * Standard modular uninstall function.
	 */
	function uninstall()
	{
		$GLOBALS['SITE_DB']->drop_if_exists('text');

		delete_config_option('system_flagrant');
	}

	/**
	 * Standard modular install function.
	 *
	 * @param  ?integer	What version we're upgrading from (NULL: new install)
	 * @param  ?integer	What hack version we're upgrading from (NULL: new-install/not-upgrading-from-a-hacked-version)
	 */
	function install($upgrade_from=NULL,$upgrade_from_hack=NULL)
	{
		if (is_null($upgrade_from))
		{
			$GLOBALS['SITE_DB']->create_table('text',array(
				'id'=>'*AUTO',
				'user_id'=>'USER',
				'the_message'=>'SHORT_TRANS',	// Comcode
				'days'=>'INTEGER',
				'order_time'=>'TIME',
				'activation_time'=>'?TIME',
				'active_now'=>'BINARY',
				'notes'=>'LONG_TEXT'
			));

			$GLOBALS['SITE_DB']->create_index('text','findflagrant',array('active_now'));

			add_config_option('SYSTEM_FLAGRANT','system_flagrant','transline','return \'\';','SITE','GENERAL');
		} else
		{
			if ($upgrade_from<3)
			{
				delete_config_option('system_flagrant');
				add_config_option('SYSTEM_FLAGRANT','system_flagrant','transline','return \'\';','SITE','GENERAL');
			}
		}
	}

	/**
	 * Standard aed_module run_start.
	 *
	 * @param  ID_TEXT		The type of module execution
	 * @return tempcode		The output of the run
	 */
	function run_start($type)
	{
		//$GLOBALS['HELPER_PANEL_PIC']='flagrant';
		$GLOBALS['HELPER_PANEL_TUTORIAL']='tut_points';

		require_lang('flagrant');
		require_code('flagrant');

		$this->add_one_label=do_lang_tempcode('ADD_FLAGRANT');
		$this->edit_this_label=do_lang_tempcode('EDIT_THIS_FLAGRANT');
		$this->edit_one_label=do_lang_tempcode('EDIT_FLAGRANT');

		if ($type=='misc') return $this->misc();

		return new ocp_tempcode();
	}

	/**
	 * The do-next manager for before content management.
	 *
	 * @return tempcode		The UI
	 */
	function misc()
	{
		require_code('templates_donext');
		return do_next_manager(get_screen_title('FLAGRANT_TEXT'),comcode_lang_string('DOC_FLAGRANT'),
					array(
						/*	 type							  page	 params													 zone	  */
						array('add_one',array('_SELF',array('type'=>'ad'),'_SELF'),do_lang('ADD_FLAGRANT')),
						array('edit_one',array('_SELF',array('type'=>'ed'),'_SELF'),do_lang('EDIT_FLAGRANT')),
					),
					do_lang('FLAGRANT_TEXT')
		);
	}

	/* *
	 * Standard aed_module preview generator.
	 *
	 * @return tempcode		The preview
	 */
	/*function get_preview()
	{
		$message=post_param('message');
		return comcode_to_tempcode($message);
	}*/

	/**
	 * Standard modular entry-point finder function.
	 *
	 * @return ?array	A map of entry points (type-code=>language-code) (NULL: disabled).
	 */
	function get_entry_points()
	{
		return array('misc'=>'FLAGRANT_MANAGE');
	}

	/**
	 * Standard aed_module table function.
	 *
	 * @param  array			Details to go to build_url for link to the next screen.
	 * @return array			A pair: The choose table, Whether re-ordering is supported from this screen.
	 */
	function nice_get_choose_table($url_map)
	{
		require_code('templates_results_table');

		$current_ordering=get_param('sort','the_message ASC');
		if (strpos($current_ordering,' ')===false) warn_exit(do_lang_tempcode('INTERNAL_ERROR'));
		list($sortable,$sort_order)=explode(' ',$current_ordering,2);
		$sortables=array(
			'the_message'=>do_lang_tempcode('MESSAGE'),
			'days'=>do_lang_tempcode('NUMBER_DAYS'),
			'order_time'=>do_lang_tempcode('ORDER_DATE'),
			'user_id'=>do_lang_tempcode('OWNER'),
		);
		if (((strtoupper($sort_order)!='ASC') && (strtoupper($sort_order)!='DESC')) || (!array_key_exists($sortable,$sortables)))
			log_hack_attack_and_exit('ORDERBY_HACK');
		global $NON_CANONICAL_PARAMS;
		$NON_CANONICAL_PARAMS[]='sort';

		$header_row=results_field_title(array(
			do_lang_tempcode('MESSAGE'),
			do_lang_tempcode('NUMBER_DAYS'),
			do_lang_tempcode('ORDER_DATE'),
			do_lang_tempcode('_UP_FOR'),
			do_lang_tempcode('OWNER'),
			do_lang_tempcode('ACTIONS'),
		),$sortables,'sort',$sortable.' '.$sort_order);

		$fields=new ocp_tempcode();

		require_code('form_templates');
		list($rows,$max_rows)=$this->get_entry_rows(false,$current_ordering);
		foreach ($rows as $row)
		{
			$edit_link=build_url($url_map+array('id'=>$row['id']),'_SELF');

			$username=protect_from_escaping($GLOBALS['FORUM_DRIVER']->member_profile_hyperlink($row['user_id']));

			$activation_time=$row['activation_time'];
			$days=is_null($activation_time)?'':float_format(round((time()-$activation_time)/60/60/24,3));

			$fields->attach(results_entry(array(protect_from_escaping(get_translated_tempcode($row['the_message'])),integer_format($row['days']),get_timezoned_date($row['order_time']),($row['active_now']==1)?$days:do_lang_tempcode('NA_EM'),$username,protect_from_escaping(hyperlink($edit_link,do_lang_tempcode('EDIT'),false,true,'#'.strval($row['id'])))),true));
		}

		return array(results_table(do_lang($this->menu_label),get_param_integer('start',0),'start',either_param_integer('max',20),'max',$max_rows,$header_row,$fields,$sortables,$sortable,$sort_order),false);
	}

	/**
	 * Get tempcode for a flagrant-message adding/editing form.
	 *
	 * @param  SHORT_TEXT	The message
	 * @param  integer		The number of days to display for
	 * @param  LONG_TEXT		Notes
	 * @param  BINARY			Whether the message is for immediate use
	 * @return tempcode		The tempcode for the fields
	 */
	function get_form_fields($message='',$days=1,$notes='',$validated=0)
	{
		$fields=new ocp_tempcode();
		require_code('form_templates');
		$fields->attach(form_input_line_comcode(do_lang_tempcode('MESSAGE'),do_lang_tempcode('DESCRIPTION_MESSAGE'),'message',$message,true));
		$fields->attach(form_input_integer(do_lang_tempcode('NUMBER_DAYS'),do_lang_tempcode('NUMBER_DAYS_DESCRIPTION'),'days',$days,true));
		if (get_value('disable_staff_notes')!=='1')
			$fields->attach(form_input_text(do_lang_tempcode('NOTES'),do_lang_tempcode('DESCRIPTION_NOTES'),'notes',$notes,false));
		$fields->attach(form_input_tick(do_lang_tempcode('IMMEDIATE_USE'),do_lang_tempcode('DESCRIPTION_IMMEDIATE_USE'),'validated',$validated==1));
		return $fields;
	}

	/**
	 * Standard aed_module edit form filler.
	 *
	 * @param  ID_TEXT		The entry being edited
	 * @return array			A quartet: fields, hidden, delete-fields, text
	 */
	function fill_in_edit_form($id)
	{
		$rows=$GLOBALS['SITE_DB']->query_select('text',array('*'),array('id'=>intval($id)));
		if (!array_key_exists(0,$rows))
		{
			warn_exit(do_lang_tempcode('MISSING_RESOURCE'));
		}
		$myrow=$rows[0];
		$date=get_timezoned_date($myrow['order_time']);
		$date_raw=$myrow['order_time'];
		$fields=$this->get_form_fields(get_translated_text($myrow['the_message']),$myrow['days'],$myrow['notes'],$myrow['active_now']);

		$username=$GLOBALS['FORUM_DRIVER']->member_profile_hyperlink($myrow['user_id']);

		$text=do_template('FLAGRANT_DETAILS',array('_GUID'=>'dcc7a8b027d450a3c17c79b23b39cd87','USERNAME'=>$username,'DAYS_ORDERED'=>integer_format($myrow['days']),'DATE_RAW'=>strval($date_raw),'DATE'=>$date));

		return array($fields,new ocp_tempcode(),new ocp_tempcode(),$text);
	}

	/**
	 * Get posted access map.
	 *
	 * @return array			A map of access permissions
	 */
	function get_permissions()
	{
		$groups=$GLOBALS['FORUM_DRIVER']->get_usergroup_list(false,true);
		$output=array();
		foreach (array_keys($groups) as $group_id)
		{
			$value=post_param_integer('access_'.$group_id,0);
			$output[$group_id]=$value;
		}

		return $output;
	}

	/**
	 * Standard aed_module add actualiser.
	 *
	 * @return ID_TEXT		The entry added
	 */
	function add_actualisation()
	{
		$message=post_param('message');
		$notes=post_param('notes','');
		$validated=post_param_integer('validated',0);

		return strval(add_flagrant($message,post_param_integer('days'),$notes,$validated));
	}

	/**
	 * Standard aed_module edit actualiser.
	 *
	 * @param  ID_TEXT		The entry being edited
	 */
	function edit_actualisation($id)
	{
		$message=post_param('message');
		$notes=post_param('notes','');
		$validated=post_param_integer('validated',0);
		edit_flagrant(intval($id),$message,$notes,$validated);
	}

	/**
	 * Standard aed_module delete actualiser.
	 *
	 * @param  ID_TEXT		The entry being deleted
	 */
	function delete_actualisation($id)
	{
		delete_flagrant(intval($id));
	}
}


